{ ------------------------------------------------------------- }
{                                                               }
{   Name:           GraphExt.pas                                }
{   Version:        V 0.62                                      }
{   Author:         Erick I. Jimenez Alvarado                   }
{   To contact:     erickj@usa.net                              }
{                   http://members.xoom.com/erickj              }
{                                                               }
{   Description:    Extended Graphics Functions                 }
{                                                               }
{   File Status:    COMPONETWARE ( like a FREEWARE but if you   }
{                   make a Componet or modify this please       }
{                   send or e-mail me this).                    }
{                                                               }
{                                                               }
{   Last Upadated:  02/05/99                                    }
{                                                               }
{   History:                                                    }
{                                                               }
{    11/22/98   V0.1 -  Internally use                          }
{    01/05/99   V0.2 -  First relase version                    }
{                       Add class TColorList                    }
{                       Add new funtion: CreateListPalette      }
{    01/08/99   V0.3 -  Remove  class TColorList and put it     }
{                       in other unit (GraphCls.pas)            }
{                       Modify function CreateListPalette       }
{    01/14/99   V0.4 -  Add new function : MergeColorExt, is    }
{                       based in code by Kerstin Thaler         }
{                                                               }
{    01/21/99   V0.5 -  Rewrite function : CreateListPalette    }
{                       Add new functions: CreateGradientColors,}
{                       and DrawGradient (Based on TGradient)   }
{                       Correct few bugs (all optimized)        }
{                                                               }
{    01/23/99   V0.6 -  Add functions: AdjRectSides,DrawFrameEx,}
{                       DrawFrameSides,DrawGradientW,DrawBorder,}
{                       DrawBorderSides,FillRectEx.             }
{    01/29/98   V0.61-  Add Bitmap Rutines. Integrate GraphCls. }
{    02/05/99   V0.62-  Add Align Styles to AlignRect,AlignBouns}
{                                                               }
{   Gratefulness: * Brad Stowers for all componets, great work  }
{                   thanks for all.                             }
{                   e-mail: bstowers@pobox.com                  }
{                   http://www.pobox.com/~bstowers/delphi/      }
{                                                               }
{                 * At RXLib Team Develope. It's the best       }
{                   Freeware Library. Thanks for all.           }
{                   Authors:                                    }
{                       Fedor Koshevnikov  (fkozh@usa.net)      }
{                       Igor Pavluk        (ipavluk@usa.net)    }
{                       Serge Korolev      (korolev@usa.net)    }
{                                                               }
{                       FidoNet: 2:5020/104                     }
{                                                               }
{                   Software development department.            }
{                       Master-Bank.                            }
{                       Moscow, Russia.                         }
{                                                               }
{                 * Kerstin Thaler for component sTktMultiBevel }
{                   thansk for all.                             }
{                   e-mail: MuK.Thaler@T-Online.de              }
{                                                               }
{   Finally:        Sorry my bad, bad english and thanks for    }
{                   use my component.                           }
{                                                               }
{ ------------------------------------------------------------- }

{$I XTools.Inc } { Directives for all controls }

unit GraphExt;

interface

uses

 {$IFNDEF WIN32}
    WinTypes, WinProcs;             { Default Units }
 {$ELSE}
    Windows,                        { Default Unit  }
 {$ENDIF}

    Graphics;

type

{
// --------------------------------------------------------
//  Based on GradientForm by Brad Stowers
//  A variant record (known as a union in C) to allow easy access to the
//  individual red, green, and blue values of a TColorRef (RGB) value.
// --------------------------------------------------------
}
  TRGBMap = packed record
    case boolean of
      TRUE:  ( RGBVal: DWORD );
      FALSE: ( Red,
               Green,
               Blue,
               Unused: Byte );
  end;

  TColorArray = Array[0..0] of TRGBMap;
  PColorArray = ^TColorArray;


{$IFDEF LIKEDCOLORS}

  {$DEFINE PRG_INTERFACE}
  {$I GraphCls.Pas}

{$ELSE}

  PColorList = ^TColorList;
  TColorList = record
    Items : Integer;
    List  : PColorArray;
  end;

{$ENDIF}  { // LIKEDCOLORS }

  { Align Types }
  TEnhAlign = ( eaTopLeft   ,aeTopCenter   ,aeTopRight,
                aeCenterLeft,aeCenter      ,aeCenterRight,
                aeBottomLeft,aeBottomCenter,aeBottomRight);

  { Sides of Frame }
  TEnhSide  = (esdLeft, esdTop, esdRight, esdBottom);
  TEnhSides = set of TEnhSide;

  TEnhFrameStyle  = (efsNone, efsLowered, efsRaised, efsLight, efsShadow);

  { Styles of Gradients }
  TGradientStyle = (gsHorizontal, gsVertical, gsTopLeft, gsTopRight,
                    gsBottomLeft, gsBottomRight, gsSquare); //, gsCustom);


const

  PaletteIndex    = $01000000;      { From Windows.pas  PaletteIndex }
  PaletteMask     = $02000000;      { From Windows.pas  PaletteRGB   }
  TransparentMask = $02000000;      { From Graphics.pas              }


  LowMaxPaletteSize = 236;          { in 256 colors the system use 20 colors }

  { Colors to optimize code }
  RGBBlackColor : TRGBMap = (RGBVal: $00000000);
  RGBWhiteColor : TRGBMap = (RGBVal: $00FFFFFF);
  RGBNavyColor  : TRGBMap = (RGBVal: $00800000);

{
// --------------------------------------------------------
//  Extended Funtions
// --------------------------------------------------------
}
function  GetSystemColorBitDepth: Integer;

{
// --------------------------------------------------------
//  Misc functions
// --------------------------------------------------------
}

procedure CreateGradientColors( LoColor, HiColor: TRGBMap; CList: PColorArray;
                                Levels : Integer
                                {$IFDEF LIKEPALETTE}; Pal: Boolean{$ENDIF});
{$IFDEF LIKEPALETTE}
function  CreateListPalette(Items : Integer; CList : PColorArray) : HPalette;
function  PaletteEntries(Palette: HPalette): Integer;
{$ENDIF} { // LIKEPALETTE }

{$IFDEF LIKEALLEFECTS}
function MergeColorExt(C1: TRGBMap; C2: TRGBMap; ColPct: Byte) : TRGBMap;
{$ENDIF} { // LIKEALLEFECTS }

function  AdjRectSides(aRect: TRect; Sides : TEnhSides; W : Integer): TRect;
function  AlignRect(SrcR, DesR: TRect; Align: TEnhAlign): TRect;
function  AlignBounds(SrcR, DesR: TRect; Align: TEnhAlign): TRect;


{
// --------------------------------------------------------
//  Draw functions
// --------------------------------------------------------
}
procedure DrawGradient(FHandle : HDC; aRect : TRect; Clist : PColorArray;
                       Levels:Integer ; Style: TGradientStyle);

{$IFDEF LIKEWAVE}
procedure DrawGradientW(FHandle : HDC; aRect : TRect; Clist : PColorArray;
                       Levels:Integer ; Waves: Integer; Style: TGradientStyle);
{$ENDIF}  { // LIKEWAVE }

function  DrawFrameEx(FHandle : HDC; aRect : TRect; Style: TEnhFrameStyle;
                      Light : TRGBMap; Shadow:TRGBMap;
                      FWidth: Integer): TRect;

function  DrawFrameSides(FHandle : HDC; aRect : TRect; Sides : TEnhSides;
                      Light : TRGBMap; Shadow:TRGBMap;
                      FWidth: Integer ; Style: TEnhFrameStyle): TRect;

function  DrawBorder(FHandle:HDC; aRect: TRect; BorderColor: TRGBMap;
                     FWidth: Integer): TRect;

function  DrawBorderSides(FHandle:HDC; aRect: TRect; Sides : TEnhSides;
                        BorderColor: TRGBMap; FWidth: Integer): TRect;

procedure FillRectEx(FHandle : HDC; aRect: TRect; Color: TColorRef);

procedure BmpTileDraw(Dest : HDC; Left, Top, Width, Height : Integer;
                      Src: HDC ; BmpW, BmpH: Integer);
procedure BmpStrechDraw(Dest : HDC; Left, Top, Width, Height : Integer;
                        Src: HDC ; BmpW, BmpH: Integer);

procedure BmpSmResizeDraw(Dest : HDC; aRect : TRect; Src: TBitmap);

{ Based on RxLib function: StretchBitmapTransparent }
procedure BmpTransparentDraw(DstDC: HDC; DstX, DstY, DstW, DstH : Integer;
          SrcDC: HDC; SrcX, SrcY, SrcW, SrcH: Integer; Palette: HPalette;
          TransparentColor: TColorRef);


{
// --------------------------------------------------------
//
// --------------------------------------------------------
}
implementation

uses
    SysUtils,
    SysUtlEx,                       { My Unit       }
    Classes;
//     Graphics;


{$IFDEF LIKEDCOLORS}

  {$UNDEF PRG_INTERFACE}
  {$DEFINE PRG_IMPLEMENTATION}
  {$I GraphCls.Pas}

{$ENDIF}  { // LIKEDCOLORS }


{
// --------------------------------------------------------
//  This function get system bits per pixel
//
//      4  = 16 Colors
//      8  = 256 Colors
//      16 = Hi Color
//      24 = True Color
//
// --------------------------------------------------------
}
function GetSystemColorBitDepth: Integer;
var
    DC: HDC;
begin

    DC := CreateDC('Display', nil, nil, nil);   { Get Handle to obtain number of bits per pixel }

  try

    Result := (GetDeviceCaps(DC, PLANES) * GetDeviceCaps(DC, BITSPIXEL));

  finally

    DeleteDC(DC);                               { Free Handle }

  End;

End;


{$IFDEF LIKEPALETTE}
{
// --------------------------------------------------------
//  Get the Number of Palette Entries
// --------------------------------------------------------
}
function PaletteEntries(Palette: HPalette): Integer;
begin
  GetObject(Palette, SizeOf(Integer), @Result);
end;

{
// --------------------------------------------------------
//  Create New Palette handle
// --------------------------------------------------------
}
function CreateListPalette(Items : Integer; CList : PColorArray ) : HPalette;
Var

  I : Integer;
  PaletteInfo: PLogPalette;
  PaletteSize: Integer;

begin

    Result := 0;

    { Check for list of colors }
    if (CList = nil) or (Items = 0) then
        Exit;

    { Create palette structure }
    PaletteSize  := SizeOf(TLogPalette) + (SizeOf(TPaletteEntry) * Items);
    GetMem(PaletteInfo, PaletteSize);       { Alloc memory to palette   }

  try

    { Generate Palette from List of colors }
    with PaletteInfo^ do
    begin

        palVersion    := $0300;             { Number of Windows Version structur }
        palNumEntries := Items;             { Number of Palette entries }

        Dec(Items);

//      { Fill all palette values }
//      for I := 0 to Items do
//          with palPalEntry[I] do
//          begin
//
//              { Specifies that the color be placed in an unused entry
//                in the system palette instead of being matched to an
//                existing color in the system palette }
//              peFlags := PC_NOCOLLAPSE;
//
//
//              { Put Color }
//              peRed   := CList[i].Red;        { Get Red Color         }
//              peGreen := CList[i].Green;      { Get Green Color       }
//              peBlue  := CList[i].Blue;       { Get Blue Color        }
//
//          end;

        { Fill all palette values }
        for I := 0 to Items do
        Begin

            { Trick : Copy ListColor as TPaletteEntry }

            palPalEntry[I] := TPaletteEntry(CList[i].RGBVal);

            { Specifies that the color be placed in an unused entry
              in the system palette instead of being matched to an
              existing color in the system palette }
            palPalEntry[I].peFlags := PC_NOCOLLAPSE;

            end;

    end;


  finally

    Result := CreatePalette(PaletteInfo^);  { return Handle of palette }

    { relassed memory }
    FreeMem(PaletteInfo, PaletteSize);

  end;

end;

{$ENDIF} { // LIKEPALETTE }


{
// --------------------------------------------------------
//  Make List of Gradient Colors
// --------------------------------------------------------
}
procedure CreateGradientColors( LoColor, HiColor: TRGBMap; CList: PColorArray;
                                Levels : Integer
                                {$IFDEF LIKEPALETTE}; Pal: Boolean{$ENDIF});
var
    I: Integer;
    RedPct, GreenPct, BluePct: Single;

{$IFDEF LIKEPALETTE}
    Msk : Byte;
{$ENDIF} { // LIKEPALETTE }

begin

    { Check for list of colors }
    if (CList = nil) or (Levels = 0) then
        Exit;

    {$IFDEF LIKEPALETTE}
    if (Pal) then
        Msk := $02   { // Equal that palette Mask }
    else
        Msk := 0;
    {$ENDIF} { // LIKEPALETTE }


    { Calc the percentage of each RGB value needed for gradient }
    Dec(Levels);
    RedPct   := (HiColor.Red   - LoColor.Red  ) / Levels;
    GreenPct := (HiColor.Green - LoColor.Green) / Levels;
    BluePct  := (HiColor.Blue  - LoColor.Blue ) / Levels;

    { Fill array of colors }
    for I := 0 to Levels do
    begin
        With CList[i] do
        Begin
            Red   := LoColor.Red   + Round(RedPct   * I);
            Green := LoColor.Green + Round(GreenPct * I);
            Blue  := LoColor.Blue  + Round(BluePct  * I);

          {$IFDEF LIKEPALETTE}
            Unused := Msk;
          {$ELSE}
            Unused := 0;
          {$ENDIF} { // LIKEPALETTE }

        end;
    end;
end;

{
// --------------------------------------------------------
// Draw Gradient
// --------------------------------------------------------
}
procedure DrawGradient(FHandle : HDC; aRect : TRect; Clist : PColorArray;
                       Levels:Integer ; Style: TGradientStyle);
Var
    OldBrush : HBrush;
    I, Delta : Integer;                 { To optimize code              }

    TmpRgn   : HRgn;

  {$IFDEF LIKEALLEFECTS}
    Width    : Integer;
    Height   : Integer;
    X1,X2,Y1,Y2  : Integer;
    StepV,                              { Vertical Steps                }
    StepH   : Single;                   { Horizontal Steps              }
  {$ELSE}

   {$IFDEF HOR_GRADIENT}
    Height   : Integer;
   {$ELSE}
    Width    : Integer;
   {$ENDIF}  { // HOR_GRADIENT }

    StepG   : Single;                   { Steps of gradient             }
  {$ENDIF}  { // LIKEALLEFECTS }

begin

    { Chect Rect }
    if isRectEmpty(aRect) then
        Exit;

    TmpRgn := CreateRectRgnIndirect(aRect);
    SelectClipRgn(HDC(Pointer(FHandle)),TmpRgn);


  {$IFDEF LIKEALLEFECTS}

    Width  := aRect.Right  - aRect.Left;
    Height := aRect.Bottom - aRect.Top;

    StepV  := Width  / Levels;                  { Width of each color band  }
    StepH  := Height / Levels;                  { Height of each color band }

    if (Style = gsHorizontal) then              { To optimize code          }
        Delta := Trunc(StepV) + 1
    else if (Style = gsVertical) then
        Delta := Trunc(StepH) + 1;

  {$ELSE}

  {$IFDEF HOR_GRADIENT}
    Height:= aRect.Bottom - aRect.Top;
    StepG := Height / Levels;                   { Width of each color band  }
  {$ELSE}
    Width := aRect.Right  - aRect.Left;
    StepG := Width / Levels;                    { Height of each color band }
  {$ENDIF}  { // HOR_GRADIENT }

    Delta := Round(StepG) + 1;                  { To optimize code          }

  {$ENDIF}  { // LIKEALLEFECTS }


    Dec(Levels);                                { Adjust Limit              }

    { Fill region }
    for I := 0 to Levels do
    begin


        { Create a brush with the appropriate color for this band }
        { Select that brush into the temporary DC. }
        OldBrush := SelectObject( HDC(Pointer(FHandle)),
                    CreateSolidBrush(COLORREF(CList[I])) );

      try

      {$IFDEF LIKEALLEFECTS}

        X1 := Trunc((I)*StepV);
        Y1 := Trunc((I)*StepH);

        if (Style = gsHorizontal) then
            Begin
                X1 := X1 + aRect.Left;
                Y1 := aRect.Top;
                X2 := Delta;
                Y2 := Height;
            End
        else if (Style =  gsVertical) then
            Begin
                X1 := aRect.Left;
                Y1 := Y1 + aRect.Top;
                X2 := Width;
                Y2 := Delta;
            End
        else
        Begin

            X2 := Width  - X1;
            Y2 := Height - Y1;

            { ** The "Case Style of" generate more code ** }
            if (Style = gsTopLeft) then
            Begin
                X1 := X1 + aRect.Left;
                Y1 := Y1 + aRect.Top;
            End
            else if (Style = gsTopRight) then
            Begin
                X1 := aRect.Left;
                Y1 := Y1 + aRect.Top;
            End
            else if (Style = gsBottomLeft) then
            Begin
                X1 := X1 + aRect.Left;
                Y1 := aRect.Top;
            End
            else if (Style = gsBottomRight) then
            Begin
                X1 := aRect.Left;
                Y1 := aRect.Top;
            End
            else if (Style = gsSquare) then
            Begin
                X1 := (X1 Shr 1) + aRect.Left;
                Y1 := (Y1 Shr 1) + aRect.Top;
            End;
        End;

//          gsCustom:
//          X1 := X1 Shr 1;             { Adjust Delta X }
//          Y1 := Y1 Shr 1;             { Adjust Delta Y }
//
//          { Hide Pen }
//          TmpPen := CreatePen(PS_NULL,0,0);
//          OldPen := SelectObject( HDC(Pointer(Handle)), TmpPen);
//
//          Ellipse(HDC(Pointer(Handle)), X1,Y2,Width - X1, Height - Y1);
//          { Restore Pen }
//          SelectObject(HDC(Pointer(Handle)), OldPen);
//          DeleteObject(TmpPen);


        { // Fill the rectangle using the selected brush    }
        { // PatBlt is faster than FillRect                 }
        PatBlt(HDC(Pointer(FHandle)),X1,Y1,X2,Y2, PATCOPY );

      {$ELSE}

        { Fill the rectangle using the selected brush }
        PatBlt(HDC(Pointer(FHandle)),

        {$IFDEF HOR_GRADIENT}
            aRect.Left + Round(I*StepG),
            aRect.Top,
            Delta,
            Height,
        {$ELSE}
            aRect.Left,
            aRect.Top + Round(I*StepG),
            Width,
            Delta,
        {$ENDIF}  { // HOR_GRADIENT }
            PATCOPY
        );

      {$ENDIF}  { // LIKEALLEFECTS }

      finally

        { Clean up the brush }
        DeleteObject(SelectObject(HDC(Pointer(FHandle)),OldBrush));

        DeleteObject(TmpRgn);
        SelectClipRgn(HDC(Pointer(FHandle)),0);

      end; { try }

    end; { For I }
end;


{$IFDEF LIKEWAVE}

{
// --------------------------------------------------------
// Draw Gradient with Waves
// --------------------------------------------------------
}
procedure DrawGradientW(FHandle : HDC; aRect : TRect; Clist : PColorArray;
                       Levels:Integer ; Waves: Integer; Style: TGradientStyle);
Var
    OldBrush : HBrush;
    I,J      : Integer;
    Delta    : Integer;                 { To optimize code              }
    LimitC   : Integer;

    TmpRgn   : HRgn;

    Width    : Integer;
    Height   : Integer;
    X1,X2,Y1,Y2  : Integer;
    StepV,                              { Vertical Steps                }
    StepH   : Single;                   { Horizontal Steps              }

begin

    { Chect Rect }
    if isRectEmpty(aRect) then
        Exit;

    TmpRgn := CreateRectRgnIndirect(aRect);
    SelectClipRgn(HDC(Pointer(FHandle)),TmpRgn);


    Width  := aRect.Right  - aRect.Left;
    Height := aRect.Bottom - aRect.Top;

    { Adjust to waves }
    LimitC := Levels - 1;
    Levels := Levels * Waves;
    J      := 0;

    StepV  := Width  / Levels;                  { Width of each color band  }
    StepH  := Height / Levels;                  { Height of each color band }

    if (Style = gsHorizontal) then              { To optimize code          }
        Delta := Trunc(StepV) + 1
    else if (Style = gsVertical) then
        Delta := Trunc(StepH) + 1;

    Dec(Levels);                                { Adjust Limit              }

    { Fill region }
    for I := 0 to Levels do
    begin


        { Create a brush with the appropriate color for this band }
        { Select that brush into the temporary DC. }
        OldBrush := SelectObject(HDC(Pointer(FHandle)),
                    CreateSolidBrush(COLORREF(CList[J])) );

      try

        X1 := Trunc((I)*StepV);
        Y1 := Trunc((I)*StepH);

        if (Style = gsHorizontal) then
            Begin
                X1 := X1 + aRect.Left;
                Y1 := aRect.Top;
                X2 := Delta;
                Y2 := Height;
            End
        else if (Style =  gsVertical) then
            Begin
                X1 := aRect.Left;
                Y1 := Y1 + aRect.Top;
                X2 := Width;
                Y2 := Delta;
            End
        else
        Begin

            X2 := Width  - X1;
            Y2 := Height - Y1;

            { ** The "Case Style of" generate more code ** }
            if (Style = gsTopLeft) then
            Begin
                X1 := X1 + aRect.Left;
                Y1 := Y1 + aRect.Top;
            End
            else if (Style = gsTopRight) then
            Begin
                X1 := aRect.Left;
                Y1 := Y1 + aRect.Top;
            End
            else if (Style = gsBottomLeft) then
            Begin
                X1 := X1 + aRect.Left;
                Y1 := aRect.Top;
            End
            else if (Style = gsBottomRight) then
            Begin
                X1 := aRect.Left;
                Y1 := aRect.Top;
            End
            else if (Style = gsSquare) then
            Begin
                X1 := (X1 Shr 1) + aRect.Left;
                Y1 := (Y1 Shr 1) + aRect.Top;
            End;
        End;


        { // Fill the rectangle using the selected brush    }
        { // PatBlt is faster than FillRect                 }
        PatBlt(HDC(Pointer(FHandle)),X1,Y1,X2,Y2, PATCOPY );

      finally

        { Clean up the brush }
        DeleteObject(SelectObject(HDC(Pointer(FHandle)),OldBrush));

        DeleteObject(TmpRgn);
        SelectClipRgn(HDC(Pointer(FHandle)),0);

        Inc(J);

        if (J > LimitC) then
            J := 0;

      end; { try }

    end; { For I }
end;

{$ENDIF}  { // LIKEWAVE }



{
// --------------------------------------------------------
//  Convert Color "C1" to "C2" in "ColPct" Percent
//  * Base on idea of Kerstin Thaler
// --------------------------------------------------------
}
function MergeColorExt(C1: TRGBMap; C2: TRGBMap; ColPct: Byte) : TRGBMap;
var
    RGBPct  : Single;

begin
    RGBPct := ColPct / 100;

    With Result do
    Begin
        Red    := C1.Red    + Round(RGBPct * (C2.Red  - C1.Red  ) );
        Green  := C1.Green  + Round(RGBPct * (C2.Green- C1.Green) );
        Blue   := C1.Blue   + Round(RGBPct * (C2.Blue - C1.Blue ) );
        Unused := 0;
    End;
end;

{
// --------------------------------------------------------
//  Adjust a Rect
// --------------------------------------------------------
}
function  AdjRectSides(aRect: TRect; Sides : TEnhSides; W : Integer): TRect;
begin

    With aRect do
    Begin
        if (esdLeft in Sides) then
        begin
            Inc(Left,W);
        end;

        if esdTop in Sides then
        begin
            Inc(Top,W);
        end;

        if esdRight in Sides then
        begin
            Dec(Right,W);
        end;

        if esdBottom in Sides then
        begin
            Dec(Bottom,W);
        end;
    end;

    Result := aRect;

end;

{
// --------------------------------------------------------
//  Center Rect
// --------------------------------------------------------
}
function  AlignRect(SrcR, DesR: TRect; Align: TEnhAlign): TRect;
Var
    DW,DH  :Integer;
    TmpAlg : Word;

begin

    With SrcR do
    Begin
        DW := Right  - DesR.Right;
        DH := Bottom - DesR.Bottom;
    End;

    TmpAlg := Word(Align);

    with Result do
    begin

        { Correct Rows }
        if (TmpAlg < 3) then                        { Top       }
            Top := 0
        else if (TmpAlg >= 3) and (TmpAlg < 6) then { Center    }
        begin
            Dec(TmpAlg,3);
            Top := DH Shr 1;
        end
        else                                        { Bottom    }
        begin
            Dec(TmpAlg,6);
            Top := DH;
        end;

        { Correct Columns }
        if (TmpAlg = 0) then                        { Left      }
            Left := 0
        else if (TmpAlg = 1) then                   { Center    }
            Left := DW Shr 1
        else                                        { Right     }
            Left := DW;

        Inc(Top, SrcR.Top);
        Inc(Left,SrcR.Left);

        Bottom := SrcR.Bottom;
        Right  := SrcR.Right;

    end;
end;

{
// --------------------------------------------------------
//  Center Bounds
// --------------------------------------------------------
}
function  AlignBounds(SrcR, DesR: TRect; Align: TEnhAlign): TRect;
Var
    DW,DH  :Integer;
    TmpAlg : Word;

begin

    With SrcR do
    Begin
        DW := (Right  - Left) - (DesR.Right  - DesR.Left);
        DH := (Bottom - Top)  - (DesR.Bottom - DesR.Top);
    End;

    TmpAlg := Word(Align);

    with Result do
    begin

        { Correct Rows }
        if (TmpAlg < 3) then                        { Top       }
            Top := 0
        else if (TmpAlg >= 3) and (TmpAlg < 6) then { Center    }
        begin
            Dec(TmpAlg,3);
            Top := DH Shr 1;
        end
        else                                        { Bottom    }
        begin
            Dec(TmpAlg,6);
            Top := DH;
        end;

        { Correct Columns }
        if (TmpAlg = 0) then                        { Left      }
            Left := 0
        else if (TmpAlg = 1) then                   { Center    }
            Left := DW Shr 1
        else                                        { Right     }
            Left := DW;

        Inc(Top, SrcR.Top);
        Inc(Left,SrcR.Left);

        Bottom := Top  + SrcR.Bottom;
        Right  := Left + SrcR.Right;
    end;
end;




{
// --------------------------------------------------------
// Draw Frame
// --------------------------------------------------------
}
function  DrawFrameEx(FHandle : HDC; aRect : TRect; Style: TEnhFrameStyle;
                      Light : TRGBMap; Shadow:TRGBMap;
                      FWidth: Integer): TRect;
Var
    OldPen    : HPen;
    TmpLight,
    TmpShadow,
    OldBrush : HBrush;

    TmpColor : TRGBMap;

    xPoint  : array[0..5] of TPoint;
    bRect   : TRect;

Begin

    { Chect Rect }
    if (FWidth > 0) and not isRectEmpty(aRect) then
    Begin

        { First check de colors }
        if (Style = efsRaised) then             { Is needed swap colors }
        Begin
            TmpColor := Light;
            Light    := Shadow;
            Shadow   := TmpColor;
        end
        else if (Style = efsLight) then         { Equal Colors }
        Begin
            Shadow   := Light;
        End
        else if (Style = efsShadow) then
        Begin
            Light    := Shadow;
        End;

        TmpLight  := CreateSolidBrush(TColorRef(Light));
        TmpShadow := CreateSolidBrush(TColorRef(Shadow));


        OldPen   := SelectObject(HDC(Pointer(FHandle)),CreatePen(PS_NULL,0,0));
        OldBrush := SelectObject(HDC(Pointer(FHandle)),TmpLight);

      try

        //With bRect do

        With aRect do
        Begin

            bRect:=Rect(Left+FWidth, Top+FWidth, Right-FWidth, Bottom-FWidth);

            XPoint[0] := Point(Left, Bottom);
            XPoint[1] := Point(bRect.Left , bRect.Bottom);
            XPoint[2] := Point(bRect.Right, bRect.Bottom);

            XPoint[3] := Point(bRect.Right, bRect.Top);
            XPoint[4] := Point(Right, Top);
            XPoint[5] := Point(Right, Bottom);

            // SelectObject(TmpLight);
            Polygon(FHandle,XPoint,6);

            SelectObject(HDC(Pointer(FHandle)),TmpShadow);

            XPoint[0] := Point(Left, Bottom);
            XPoint[1] := Point(bRect.Left, bRect.Bottom);
            XPoint[2] := Point(bRect.Left, bRect.Top);

            XPoint[3] := Point(bRect.Right, bRect.Top);
            XPoint[4] := Point(Right, Top);
            XPoint[5] := Point(Left, Top);
            Polygon(FHandle,XPoint,6);
        End;

      finally

        DeleteObject(SelectObject(HDC(Pointer(FHandle)),OldPen));
        SelectObject(HDC(Pointer(FHandle)),OldBrush);

        DeleteObject(TmpLight);
        DeleteObject(TmpShadow);

        InflateRect(aRect,-FWidth,-FWidth);

      end;

    end;

    Result := aRect;
(*
    efsLowered2:   // Like a win control
        DrawEdge(, aRect, EDGE_SUNKEN, BF_RECT or BF_SOFT or BF_ADJUST);
    efsRaised2:    // Like a win control
        DrawEdge(, aRect, EDGE_RAISED, BF_RECT or BF_SOFT or BF_ADJUST);
    efsLowered:
        DrawEdge(HDC(Pointer(FHandle)), aRect, BDR_RAISEDINNER, BF_RECT or flags);
    efsRaised:
        DrawEdge(HDC(Pointer(FHandle)), aRect, BDR_SUNKENOUTER, BF_RECT or flags);

    efsWindowBorder:
      begin
        DrawEdge(HDC(Pointer(FHandle)), aRect, BDR_RAISEDOUTER, BF_BOTTOMRIGHT or flags);
        inc(aRect.Top);
        inc(aRect.Left);
        DrawEdge(HDC(Pointer(FHandle)), aRect, BDR_RAISEDINNER, BF_RECT or flags);
      end;
*)
End;


{
// --------------------------------------------------------
// Draw Frame by Sides
// --------------------------------------------------------
}
function DrawFrameSides(FHandle : HDC; aRect : TRect; Sides : TEnhSides;
                      Light : TRGBMap; Shadow:TRGBMap;
                      FWidth: Integer ; Style: TEnhFrameStyle): TRect;

Var
    OldPen    : HPen;

    TmpLight,
    TmpShadow,
    OldBrush  : HBrush;

    TmpColor : TRGBMap;

    bRect   : TRect;
    xPoint  : array[0..3] of TPoint;

Begin

    Result := aRect;

    { Chect Rect }
    if (FWidth > 0) and not isRectEmpty(aRect) then
    Begin

        { First check de colors }
        if (Style = efsRaised) then             { Is needed swap colors }
        Begin
            TmpColor := Light;
            Light    := Shadow;
            Shadow   := TmpColor;
        end
        else if (Style = efsLight) then         { Equal Colors }
        Begin
            Shadow   := Light;
        End
        else if (Style = efsShadow) then
        Begin
            Light    := Shadow;
        End;


        TmpLight  := CreateSolidBrush(TColorRef(Light));
        TmpShadow := CreateSolidBrush(TColorRef(Shadow));

        OldPen   := SelectObject(HDC(Pointer(FHandle)),CreatePen(PS_NULL,0,0));
        OldBrush := SelectObject(HDC(Pointer(FHandle)),TmpLight);

      try

        With aRect do
        Begin

            bRect := AdjRectSides(aRect,Sides,FWidth);

            if (esdBottom in Sides) then
            Begin
                XPoint[0] := Point(Left, Bottom);
                XPoint[1] := Point(bRect.Left , bRect.Bottom);
                XPoint[2] := Point(bRect.Right, bRect.Bottom);
                XPoint[3] := Point(Right, Bottom);
                Polygon(FHandle,XPoint,4);
                Dec(Result.Bottom,FWidth);
            End;

            if (esdRight in Sides) then
            Begin
                XPoint[0] := Point(bRect.Right, bRect.Bottom);
                XPoint[1] := Point(bRect.Right, bRect.Top);
                XPoint[2] := Point(Right, Top);
                XPoint[3] := Point(Right, Bottom);
                Polygon(FHandle,XPoint,4);
                Dec(Result.Right,FWidth);
            End;


            { Select Sadow Color }
            SelectObject(HDC(Pointer(FHandle)),TmpShadow);

            if (esdLeft   in Sides) then
            Begin
                XPoint[0] := Point(Left, Bottom);
                XPoint[1] := Point(bRect.Left , bRect.Bottom);
                XPoint[2] := Point(bRect.Left , bRect.Top);
                XPoint[3] := Point(LEft, Top);
                Polygon(FHandle,XPoint,4);
                Inc(Result.Left,FWidth);
            End;

            if (esdTop   in Sides) then
            Begin
                XPoint[0] := Point(bRect.Left, bRect.Top);
                XPoint[1] := Point(Left, Top);
                XPoint[2] := Point(Right, Top);
                XPoint[3] := Point(bRect.Right,bRect.Top);
                Polygon(FHandle,XPoint,4);
                Inc(Result.Top,FWidth);
            End;

        End;

      finally

        DeleteObject(SelectObject(HDC(Pointer(FHandle)),OldPen));
        SelectObject(HDC(Pointer(FHandle)),OldBrush);

        DeleteObject(TmpLight);
        DeleteObject(TmpShadow);

        InflateRect(aRect,-FWidth,-FWidth);

      end;

    end;
end;



{
// --------------------------------------------------------
// Draw Border in Rect
// --------------------------------------------------------
}
function DrawBorder(FHandle: HDC; aRect: TRect; BorderColor: TRGBMap;
                     FWidth: Integer): TRect;

Var
    OldPen   : HPen;
    OldBrush : HBrush;
    TmpBrush : TLogBrush;

begin

    if (FWidth > 0) and not isRectEmpty(aRect) then
    begin

        TmpBrush.lbStyle := BS_HOLLOW;
    //    TmpBrush.lbColor := 0;

        OldBrush := SelectObject(HDC(Pointer(FHandle)),
                    CreateBrushIndirect(TmpBrush));


        OldPen := SelectObject(HDC(Pointer(FHandle)),
                  CreatePen(PS_SOLID,1,TColorRef(BorderColor)));

      try

        With aRect do
        begin
            while FWidth > 0 do
            begin
                Rectangle(HDC(Pointer(FHandle)),Left,Top,Right,Bottom);
                Inc(Left);
                Inc(Top);
                Dec(Right);
                Dec(Bottom);
                Dec(FWidth);
            end;

        end;

      finally

        DeleteObject(SelectObject(HDC(Pointer(FHandle)),OldPen));
        DeleteObject(SelectObject(HDC(Pointer(FHandle)),OldBrush));

      end;

    end;

    Result := aRect;
end;

{
// --------------------------------------------------------
// Draw Border in Rect Custom
// --------------------------------------------------------
}
function  DrawBorderSides(FHandle: HDC; aRect: TRect; Sides : TEnhSides;
                        BorderColor: TRGBMap; FWidth: Integer): TRect;

Var
    OldPen   : HPen;
    OldBrush : HBrush;
    TmpBrush : TLogBrush;

{ Draws a frame with the specified width and colors of frame sides.
  Sides parameter specifies which sides of frame will be drawn. }

  procedure DrawRectSides;
  { Draws the sides of Rect if necessary }
  begin
    with  aRect do
    begin
        if (esdLeft in Sides) then
        begin
            MoveToEx(HDC(Pointer(FHandle)), Left, Top, nil);
            LineTo(HDC(Pointer(FHandle)), Left, Bottom);
            Inc(Left);
        end;
        if esdTop in Sides then
        begin
            MoveToEx(HDC(Pointer(FHandle)),Left, Top, nil);
            LineTo(HDC(Pointer(FHandle)), Right, Top);
            Inc(Top);
        end;

        if esdRight in Sides then
        begin
            MoveToEx(HDC(Pointer(FHandle)), Right-1, Top, nil);
            LineTo(HDC(Pointer(FHandle)), Right-1, Bottom);
            Dec(Right);
        end;
        if esdBottom in Sides then
        begin
            MoveToEx(HDC(Pointer(FHandle)), Left, Bottom-1, nil);
            LineTo(HDC(Pointer(FHandle)), Right, Bottom-1);
            Dec(Bottom);
        end;
    end;
  end;

begin

    if (FWidth > 0) and not isRectEmpty(aRect) then
    begin
        TmpBrush.lbStyle := BS_HOLLOW;

        OldBrush := SelectObject(HDC(Pointer(FHandle)),
                    CreateBrushIndirect(TmpBrush));


        OldPen := SelectObject(HDC(Pointer(FHandle)),
                  CreatePen(PS_SOLID,1,TColorRef(BorderColor)));

      try

        while FWidth > 0 do
        begin
            DrawRectSides;
            Dec(FWidth);
        end;


      finally

        DeleteObject(SelectObject(HDC(Pointer(FHandle)), OldPen));
        DeleteObject(SelectObject(HDC(Pointer(FHandle)), OldBrush));

      end;

    end;

    Result := aRect;
end;



{
// --------------------------------------------------------
//  FillRectEx
// --------------------------------------------------------
}
procedure FillRectEx(FHandle : HDC; aRect: TRect; Color: TColorRef);
Var
    OldPen   : HPen;
    OldBrush : HBrush;

Begin

    OldBrush := SelectObject(HDC(Pointer(FHandle)),
                CreateSolidBrush(TColorRef(Color)));

    OldPen   := SelectObject(HDC(Pointer(FHandle)),
                CreatePen(PS_NULL,0,0));

  try

    With aRect do
        PatBlt(HDC(Pointer(FHandle)),Left,Top,Right-Left,Bottom-Left,PATCOPY);

  finally

    DeleteObject(SelectObject(HDC(Pointer(FHandle)), OldPen));
    DeleteObject(SelectObject(HDC(Pointer(FHandle)), OldBrush));

  end;

End;


{
// --------------------------------------------------------
// Paint Title Bitmap
// --------------------------------------------------------
}
procedure BmpTileDraw(Dest : HDC; Left, Top, Width, Height : Integer;
                      Src: HDC ; BmpW, BmpH: Integer);
Var

    I : Integer;
    TmpRgn : HRgn;

begin

    { Chect Rect }
    if (BmpW = 0) or (BmpH = 0) then
        Exit;

    TmpRgn := CreateRectRgnIndirect(Rect(Left,Top,Left+Width,Top+Height));

    SelectClipRgn(HDC(Pointer(Dest)),TmpRgn);

  try

    BitBlt(HDC(Pointer(Dest)), Left, Top, BmpW, BmpH,
           HDC(Pointer(Src)),0,0,SRCCOPY);

    I := 0;

    While (I < Width) do
    begin
        BitBlt(HDC(Pointer(Dest)),Left+I, Top, BmpW, BmpH,
               HDC(Pointer(Dest)),Left,Top,SRCCOPY);
        Inc(I,BmpW);
//        Inc(BmpW,BmpW);
    end;

    I := BmpH;

    While (I < Height) do
    begin
        BitBlt(HDC(Pointer(Dest)), Left, Top+I, Width, BmpH,
               HDC(Pointer(Dest)),Left,Top,SRCCOPY);
        Inc(I,BmpH);
//        Inc(BmpH,BmpH);
    end;

  finally

    DeleteObject(TmpRgn);
    SelectClipRgn(HDC(Pointer(Dest)),0);

  end;

End;


{
// --------------------------------------------------------
// Paint Strech Bitmap
// --------------------------------------------------------
}
procedure BmpStrechDraw(Dest : HDC; Left, Top, Width, Height : Integer;
                        Src: HDC ; BmpW, BmpH: Integer);
Var

    OldStrech : Integer;

begin

    { Chect Rect }
    if (BmpW = 0) or (BmpH = 0) then
        Exit;

    OldStrech := SetStretchBltMode(HDC(Pointer(Dest)), STRETCH_DELETESCANS);
    StretchBlt(HDC(Pointer(Dest)), Left, Top, Width, Height,
               HDC(Pointer(Src)), 0, 0, BmpW, BmpH, SRCCOPY);

    SetStretchBltMode(HDC(Pointer(Dest)), OldStrech);
End;


{
// --------------------------------------------------------
// Paint Smmoth Resize Bitmap
// --------------------------------------------------------
}
procedure BmpSmResizeDraw(Dest : HDC; aRect : TRect; Src: TBitmap);
begin
     // Not Implement for a moment
End;


{
// --------------------------------------------------------
//  Draw a Transparent Bitmap Based on RxLib function:
//  StretchBitmapTransparent
// --------------------------------------------------------
}
procedure BmpTransparentDraw(DstDC: HDC; DstX, DstY, DstW, DstH : Integer;
          SrcDC: HDC; SrcX, SrcY, SrcW, SrcH: Integer; Palette: HPalette;
          TransparentColor: TColorRef);
var

  Color: TColorRef;

  bmAndBack, bmAndObject,
  bmAndMem,  bmSave: HBitmap;
  bmBackOld, bmObjectOld,
  bmMemOld,  bmSaveOld: HBitmap;
  MemDC, BackDC,
  ObjectDC, SaveDC: HDC;
  palDst, palMem,
  palSave, palObj: HPalette;

begin

    { Create some DCs to hold temporary data }
    BackDC   := CreateCompatibleDC(HDC(Pointer(DstDC)));
    ObjectDC := CreateCompatibleDC(HDC(Pointer(DstDC)));
    MemDC    := CreateCompatibleDC(HDC(Pointer(DstDC)));
    SaveDC   := CreateCompatibleDC(HDC(Pointer(DstDC)));

    { Create a bitmap for each DC }
    bmAndObject := CreateBitmap(SrcW, SrcH, 1, 1, nil);
    bmAndBack   := CreateBitmap(SrcW, SrcH, 1, 1, nil);
    bmAndMem    := CreateCompatibleBitmap(HDC(Pointer(DstDC)), DstW, DstH);
    bmSave      := CreateCompatibleBitmap(HDC(Pointer(DstDC)), SrcW, SrcH);

    { Each DC must select a bitmap object to store pixel data }
    bmBackOld   := SelectObject(HDC(Pointer(BackDC)), bmAndBack);
    bmObjectOld := SelectObject(HDC(Pointer(ObjectDC)), bmAndObject);
    bmMemOld    := SelectObject(HDC(Pointer(MemDC)), bmAndMem);
    bmSaveOld   := SelectObject(HDC(Pointer(SaveDC)), bmSave);

    { Select palette }
    palDst := 0;
    palMem := 0;
    palSave:= 0;
    palObj := 0;

    if (Palette <> 0) then
    begin

      palDst  := SelectPalette(HDC(Pointer(DstDC)), Palette, True);
      RealizePalette(HDC(Pointer(DstDC)));

      palSave := SelectPalette(HDC(Pointer(SaveDC)), Palette, False);
      RealizePalette(HDC(Pointer(SaveDC)));

      palObj := SelectPalette(HDC(Pointer(ObjectDC)), Palette, False);
      RealizePalette(HDC(Pointer(ObjectDC)));

      palMem := SelectPalette(HDC(Pointer(MemDC)), Palette, True);
      RealizePalette(HDC(Pointer(MemDC)));

    end;

  try

    { Set proper mapping mode }
    SetMapMode(HDC(Pointer(SrcDC)),  GetMapMode(HDC(Pointer(DstDC))));
    SetMapMode(HDC(Pointer(SaveDC)), GetMapMode(HDC(Pointer(DstDC))));

    { Save the bitmap sent here }
    BitBlt(HDC(Pointer(SaveDC)), 0, 0, SrcW, SrcH,
           HDC(Pointer(SrcDC)), SrcX, SrcY, SRCCOPY);

    { Set the background color of the source DC to the color,         }
    { contained in the parts of the bitmap that should be transparent }
    Color := SetBkColor(HDC(Pointer(SaveDC)), TransparentColor or PaletteMask);

    { Create the object mask for the bitmap by performing a BitBlt()  }
    { from the source bitmap to a monochrome bitmap                   }
    BitBlt(HDC(Pointer(ObjectDC)), 0, 0, SrcW, SrcH,
           HDC(Pointer(SaveDC)), 0, 0, SRCCOPY);

    { Set the background color of the source DC back to the original  }
    SetBkColor(HDC(Pointer(SaveDC)), Color);

    { Create the inverse of the object mask }
    BitBlt(HDC(Pointer(BackDC)), 0, 0, SrcW, SrcH,
           HDC(Pointer(ObjectDC)), 0, 0, NOTSRCCOPY);

    { Copy the background of the main DC to the destination }
    BitBlt(HDC(Pointer(MemDC)), 0, 0, DstW, DstH,
           HDC(Pointer(DstDC)), DstX, DstY, SRCCOPY);

    { Mask out the places where the bitmap will be placed }
    BitBlt(HDC(Pointer(MemDC)), 0, 0, DstW, DstH,
           HDC(Pointer(ObjectDC)), 0, 0, SRCAND); // Strech

    { Mask out the transparent colored pixels on the bitmap }
    BitBlt(HDC(Pointer(SaveDC)), 0, 0, SrcW, SrcH,
           HDC(Pointer(BackDC)), 0, 0, SRCAND);


    { XOR the bitmap with the background on the destination DC }
    BitBlt(HDC(Pointer(MemDC)), 0, 0, DstW, DstH,
           HDC(Pointer(SaveDC)), 0, 0, SRCPAINT); // Strech

    { Copy the destination to the screen }
    BitBlt(HDC(Pointer(DstDC)), DstX, DstY, DstW, DstH,
           HDC(Pointer(MemDC)), 0, 0, SRCCOPY);

  finally

    { Restore palette }
    if (Palette <> 0) then
    begin
        SelectPalette(HDC(Pointer(MemDC)),    palMem,  False);
        SelectPalette(HDC(Pointer(ObjectDC)), palObj,  False);
        SelectPalette(HDC(Pointer(SaveDC)),   palSave, False);
        SelectPalette(HDC(Pointer(DstDC)),    palDst,  True);
    end;


    { Delete the memory bitmaps }
    DeleteObject(SelectObject(HDC(Pointer(BackDC)), bmBackOld));
    DeleteObject(SelectObject(HDC(Pointer(ObjectDC)), bmObjectOld));
    DeleteObject(SelectObject(HDC(Pointer(MemDC)), bmMemOld));
    DeleteObject(SelectObject(HDC(Pointer(SaveDC)), bmSaveOld));

    { Delete the memory DCs }
    DeleteDC(HDC(Pointer(MemDC)));
    DeleteDC(HDC(Pointer(BackDC)));
    DeleteDC(HDC(Pointer(ObjectDC)));
    DeleteDC(HDC(Pointer(SaveDC)));

  end;

end;

End.
